(define selfmenuroot "<Image>/contributed")     ; Change these to suit yourself         
(define selfmenuentry "Test For Duplicate Scripts...")

(define selfauthor "Kevin Payne")
(define selfcopyright "Copyright (C) 2011 Kevin Payne paynekj@hotmail.com")

;
; Version 1.0 09.11.2011 First version
; Version 1.1 10.11.2011 Add option to search all script directories
(define selffileversion "10.11.2011 Version 1.1 - Search all script directories")

(define selffilename "\n- kp24_test_for_duplicate_scripts.scm")

(define selftipstrip (string-append "Check for duplicate scripts" "\n\n\nRegisters in menu " selfmenuroot  "/" selfmenuentry "\n\nScript File name - " selffilename))


; Created this just to make the main procedure managable
(define (buildScriptFileList check_used explicit_directory)
  (let* (
           (varFileList '())
           (source_directory_list '())
        )
        (if (= check_used TRUE)
          (begin                                     ; check all the user defined 
            (catch ()    
              (set! source_directory_list (car (gimp-gimprc-query "script-fu-path")))  ; string containing the users script directories
            )
            (if (> (string-length source_directory_list) 0)
              (set! source_directory_list (strbreakup source_directory_list ";"))  ; turn the string into a list
              (set! source_directory_list (list explicit_directory))               ; no user defined script directories so make a single item list of the requested directory
            )
          )
          (set! source_directory_list (list explicit_directory)) ; don't check all, just check the requested directory
        )
        (for-each                       ; scan through the directories building a list of script files
          (lambda (directory_path)
            (set! varFileList (append varFileList (cadr (file-glob (string-append directory_path DIR-SEPARATOR "*.scm") 1) ) ) ); Get the list of script files 
          ) source_directory_list
        )
        varFileList
  )
)

(define (script-fu-kp24_test_for_duplicate_scripts check_used explicit_directory)
  (let* ( 
           (found 0) (found_remainder '()) (list_ref 0) (register_count 0) (file_count 0) (number_of_files 0)
           (token "")
           (stream 0)
           (proc_name_list '()) (proc_name "")
           (filepath_list '())
           (varFileList '())
        )

        (gimp-progress-init "Building list of files" -1)
        (set! varFileList (buildScriptFileList check_used explicit_directory)) ; Get the list of script files 
        (set! number_of_files (length varFileList))

        (for-each                      ; step through each of the script files
          (lambda (filepath)
            (set! found 0)                            ; flag to indicate if we've found what what we're looking for
            (set! stream (open-input-file filepath))  ; open the script file
            (set! file_count (+ file_count 1))
            (gimp-progress-update (/ file_count number_of_files))           ; keep the user informed
            (gimp-progress-set-text (string-append "Processing " filepath))
            (while (= found 0)                        ; loop round until we've found what what we're looking for
              (begin
                (set! token (read stream))            ; read a section from the script file
                (if (list? token)                     ; if we've got something useful
                  (if (string=? (symbol->string (car token)) "script-fu-register")  ; check if this section is what we want
                    (begin
                      (set! register_count (+ register_count 1))
                      (set! proc_name (car (cdr token)))                                     ; extract the procedure name
                      (if (symbol? proc_name) (set! proc_name (symbol->string proc_name))  ) ; make sure we've got a string
                      (set! found_remainder (member proc_name proc_name_list))               ; is this procedure already in the list?
                      (if (list? found_remainder)
                        (begin                                                                   ; Yes, we've found this procedure before
                          (set! list_ref (- (length proc_name_list) (length found_remainder)))   ; find the list index of the previous occurence
                          (gimp-message (string-append "duplicate: " proc_name " in \n" filepath "\n and \n" (list-ref filepath_list list_ref) )) ; Already found this name
                        )
                        (begin                                                           ; we didn't find the procedure before
                          (set! proc_name_list (append proc_name_list (list proc_name))) ; New name so add to list
                          (set! filepath_list (append filepath_list (list filepath)))    ; remember which file it was in
                        )
                      )
                    )
                  )
                  (set! found 1) ; We ran out of file
                )
              )
            )
            (close-input-port stream)  ; close the script file
          )
          varFileList
        )
        (gimp-message (string-append "Finished. Found " (number->string register_count) " register attempts"))
  )
)

(script-fu-register "script-fu-kp24_test_for_duplicate_scripts"
                    selfmenuentry
                    selftipstrip
                    selfcopyright
                    selfauthor
                    selffileversion
                    ""
                    SF-TOGGLE "Check all used directories?" TRUE
                    SF-DIRNAME "Directory" (string-append gimp-directory DIR-SEPARATOR "scripts")
                    )

(script-fu-menu-register "script-fu-kp24_test_for_duplicate_scripts" selfmenuroot)
